package com.zanxiang.sdk.service.Impl;

import com.zanxiang.common.domain.ResultVo;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.text.UUID;
import com.zanxiang.common.utils.IpUtils;
import com.zanxiang.common.utils.JsonUtil;
import com.zanxiang.common.utils.URIUtil;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.sdk.common.constant.ApiUrlConstant;
import com.zanxiang.sdk.common.constant.RedisKeyConstant;
import com.zanxiang.sdk.common.util.RedisUtils;
import com.zanxiang.sdk.domain.dto.UserOauthDTO;
import com.zanxiang.sdk.domain.params.QqLoginCallbackParam;
import com.zanxiang.sdk.domain.vo.UserLoginVO;
import com.zanxiang.sdk.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

/**
 * @author : lingfeng
 * @time : 2022-06-20
 * @description : 用户第三方授权
 */
@Slf4j
@Service
public class UserAuthServiceImpl implements IUserAuthService {

    @Autowired
    private RedisUtils<String> redisUtils;

    @Autowired
    private IQqApiService qqApiService;

    @Autowired
    private IVxApiService vxApiService;

    @Autowired
    private IUserOauthService userOauthService;

    @Autowired
    private IUserTokenService userTokenService;

    @Autowired
    private IUserLoginLogService userLoginLogService;

    @Autowired
    private IUserService userService;

    /**
     * QQ开发者应用id
     */
    @Value("${auth.qq-appId}")
    private String qqAppId;

    /**
     * QQ开发者应用id
     */
    @Value("${auth.vx-appId}")
    private String vxAppId;

    /**
     * QQ授权登录
     *
     * @param response : 返回体
     */
    @Override
    public void qqLoginAuth(HttpServletResponse response) {
        //获取state
        String state = this.getState();
        //传递参数
        Map<String, String> paramMap = new HashMap<>(4);
        paramMap.put("response_type", "code");
        paramMap.put("client_id", qqAppId);
        paramMap.put("state", state);
        paramMap.put("redirect_uri", ApiUrlConstant.QQ_REDIRECT_URL);
        //重定向到QQ授权页面
        try {
            response.sendRedirect(URIUtil.fillUrlParams(ApiUrlConstant.QQ_AUTH_URL, paramMap, Boolean.TRUE));
        } catch (Exception e) {
            log.error("QQ授权登录重定向跳转异常");
            throw new BaseException("QQ授权登录重定向跳转异常");
        }
    }

    /**
     * QQ授权登录回调
     *
     * @param qqLoginCallbackParam : 回调参数
     * @param request              : request
     * @return : 返回登录token
     */
    @Override
    public ResultVo<UserLoginVO> qqLoginCallback(QqLoginCallbackParam qqLoginCallbackParam, HttpServletRequest request) {
        String code = qqLoginCallbackParam.getCode();
        String state = qqLoginCallbackParam.getState();
        String deviceType = qqLoginCallbackParam.getDeviceType();
        //验证state，如果不一致，可能被CSRF攻击
        this.checkState(state);
        //获取用户信息
        Map<String, String> userInfoMap = qqApiService.qqAuthUserInfo(code);
        //查询用户授权信息是否存在
        UserOauthDTO userOauthDTO = userOauthService.getUserOauthByOpenId(userInfoMap.get("openId"));
        //获取用户信息
        User user = this.getUserByUserOauth(deviceType, userInfoMap, userOauthDTO);
        //用户登录成功
        String userToken = userTokenService.getUserToken(userOauthDTO.getId(), deviceType);
        //登录的ip
        String realIp = IpUtils.getRealIp(request);
        //插入用户登录记录
        userLoginLogService.addUserLoginLog(realIp, user, qqLoginCallbackParam.getGameId());
        //移出state
        redisUtils.removeOfSet(RedisKeyConstant.AUTH_STATE_KEY, state);
        //构造返回
        return new ResultVo<>(new UserLoginVO(userToken));
    }

    /**
     * 微信授权登录
     *
     * @param response : 返回体
     */
    @Override
    public void vxLoginAuth(HttpServletResponse response) {
        //获取state
        String state = this.getState();
        //传递参数
        Map<String, String> paramMap = new HashMap<>(6);
        paramMap.put("appid", qqAppId);
        paramMap.put("redirect_uri", URIUtil.encodeURIComponent(ApiUrlConstant.VX_REDIRECT_URL));
        paramMap.put("response_type", "code");
        paramMap.put("scope", "snsapi_login");
        paramMap.put("state", state);
        paramMap.put("lang", "cn");
        //重定向到QQ授权页面
        try {
            response.sendRedirect(URIUtil.fillUrlParams(ApiUrlConstant.VX_AUTH_URL, paramMap, Boolean.TRUE));
        } catch (Exception e) {
            log.error("微信授权登录重定向跳转异常");
            throw new BaseException("微信授权登录重定向跳转异常");
        }
    }

    /**
     * 微信回调方法
     *
     * @param qqLoginCallbackParam : 回调参数
     * @param request              : request
     * @return : 返回登录token
     */
    @Override
    public ResultVo<UserLoginVO> vxLoginCallback(QqLoginCallbackParam qqLoginCallbackParam, HttpServletRequest request) {
        String deviceType = qqLoginCallbackParam.getDeviceType();
        String code = qqLoginCallbackParam.getCode();
        String state = qqLoginCallbackParam.getState();
        //验证state，如果不一致，可能被CSRF攻击
        this.checkState(state);
        //获取用户信息
        Map<String, String> userInfoMap = vxApiService.vxAuthUserInfo(code);
        //查询用户授权信息是否存在
        UserOauthDTO userOauthDTO = userOauthService.getUserOauthByOpenId(userInfoMap.get("openId"));
        //获取用户信息
        User user = this.getUserByUserOauth(deviceType, userInfoMap, userOauthDTO);
        //登录的ip
        String realIp = IpUtils.getRealIp(request);
        //用户登录成功
        String userToken = userTokenService.getUserToken(userOauthDTO.getId(), deviceType);
        //插入用户登录记录
        userLoginLogService.addUserLoginLog(realIp, user, qqLoginCallbackParam.getGameId());
        //移出state
        redisUtils.removeOfSet(RedisKeyConstant.AUTH_STATE_KEY, state);
        //构造返回
        return new ResultVo<>(new UserLoginVO(userToken));
    }

    /**
     * 格局用户授权信息获取用户
     *
     * @param deviceType   : 设备类型
     * @param userInfoMap  : 第三方用户信息
     * @param userOauthDTO : 用户授权信息
     * @return : 返回应用用户信息
     */
    private User getUserByUserOauth(String deviceType, Map<String, String> userInfoMap, UserOauthDTO userOauthDTO) {
        if (userOauthDTO != null) {
            return userService.getById(userOauthDTO.getUserId());
        }
        //创建用户信息
        User user = User.builder()
                .nickname(userInfoMap.get("nickname"))
                .avatar(userInfoMap.get("avatar"))
                .deviceType(deviceType)
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .build();
        userService.save(user);
        //创建用户授权信息
        userOauthDTO = userOauthService.createUserOauth(user, userInfoMap);
        log.info("用户授权信息, userOauthDTO : {}", JsonUtil.toString(userOauthDTO));
        //返回用户信息
        return user;
    }

    /**
     * 生成随机密钥并添加到redis中
     *
     * @return : 返回随机密钥
     */
    private String getState() {
        String state = UUID.randomUUID().toString();
        redisUtils.addToSet(RedisKeyConstant.AUTH_STATE_KEY, state);
        return state;
    }

    /**
     * state参数检测
     *
     * @param state : 随机密钥
     */
    private void checkState(String state) {
        if (!redisUtils.isMemberInSet(RedisKeyConstant.AUTH_STATE_KEY, state)) {
            throw new BaseException("State验证失败");
        }
    }
}
